"""
Functional tests for the user authentication feature.

:copyright: Copyright 2013-2016, Dorian Pula <dorian.pula@amber-penguin-software.ca>
:license: AGPL v3+
"""

import collections
import http

import jwt
import pytest
import pytest_bdd as bdd
import requests
from pytest_bdd import parsers

from tests import utils


Credentials = collections.namedtuple('Credentials', ['username', 'password'])

INVALID_USER_INVALID_CREDENTIALS = Credentials('invalid_user', 'invalid_password')
VALID_USER_VALID_CREDENTIALS = Credentials('login_test_user', 'login_test_password')
VALID_USER_INVALID_CREDENTIALS = Credentials('login_test_user', 'wrong_password')


@pytest.fixture(scope='module')
def valid_test_user(db_engine):
    return utils.create_test_user(
        db_engine=db_engine,
        username=VALID_USER_VALID_CREDENTIALS.username,
        password=VALID_USER_VALID_CREDENTIALS.password)


@bdd.scenario('user_authentication.feature', 'Valid user can authenticate against site with valid credentials')
def test_authentication_with_valid_credentials():
    pass


@bdd.scenario('user_authentication.feature', 'Valid user cannot authenticate against site with invalid credentials')
def test_authentication_with_invalid_credentials():
    pass


@bdd.scenario('user_authentication.feature', 'Invalid user cannot authenticate against site with credentials')
def test_authentication_with_invalid_user():
    pass


@bdd.given(parsers.parse('I am a {user_type} user with {login_type} credentials'))
def user_credentials(user_type, login_type, valid_test_user):
    if not user_type == 'valid':
        return INVALID_USER_INVALID_CREDENTIALS

    if not login_type == 'valid':
        return VALID_USER_INVALID_CREDENTIALS

    return VALID_USER_VALID_CREDENTIALS


@bdd.given('I post my credentials against the auth endpoint')
def auth_endpoint_response(user_credentials: Credentials, api_base_uri):
    return requests.post(f'{api_base_uri}/auth', json={
        'username': user_credentials.username,
        'password': user_credentials.password,
    })


@bdd.then('I get a valid JWT')
def assert_valid_user_json_web_token(user_credentials: Credentials, auth_endpoint_response: requests.Response):
    assert auth_endpoint_response.status_code == http.HTTPStatus.OK

    jwt_token = auth_endpoint_response.json()['access_token']
    assert jwt_token is not None
    decoded_jwt = jwt.decode(jwt_token, verify=False)
    assert 'identity' in decoded_jwt

    assert auth_endpoint_response.json().get('username') == user_credentials.username


@bdd.then('I get an unauthorized response')
def assert_unauthorized_response(auth_endpoint_response: requests.Response):
    assert auth_endpoint_response.status_code == http.HTTPStatus.UNAUTHORIZED
    assert auth_endpoint_response.json()['status_code'] == http.HTTPStatus.UNAUTHORIZED
    assert auth_endpoint_response.json()['error'] == 'Bad Request'
    assert auth_endpoint_response.json()['description'] == 'Invalid credentials'
