"""
Functional tests for the managing users.

:copyright: Copyright 2013-2016, Dorian Pula <dorian.pula@amber-penguin-software.ca>
:license: AGPL v3+
"""

import http
from unittest import mock

import pytest_bdd as bdd
import requests
from pytest_bdd import parsers

from rookeries.users import models
from tests import utils

bdd.scenarios('user_management.feature')

TEST_USER_PASSWORDS = 'password-testing'
TEST_NAME_MODIFIER = ' modified'

USER_TEMPLATES = {
    'requester': {
        'admin': {
            'username': 'site-admin',
            'role': models.UserRole.admin.name,
            'name': 'Admin',
            'email': 'site-admin@test.rookeries.org',
        },
        'editor': {
            'username': 'site-editor',
            'role': models.UserRole.admin.editor.name,
            'name': 'Editor',
            'email': 'site-editor@test.rookeries.org',
        },
        'subscriber': {
            'username': 'site-member',
            'role': models.UserRole.admin.subscriber.name,
            'name': 'Member',
            'email': 'member@example.com',
        },
    },
    'target_data': {
        'admin': {
            'username': 'test-admin',
            'role': models.UserRole.admin.name,
            'name': 'Test Admin',
            'email': 'test-admin@penguin-news.net',
        },
        'editor': {
            'username': 'test-editor',
            'role': models.UserRole.admin.editor.name,
            'name': 'Main Editor',
            'email': 'editor@penguin-news.net',
        },
        'subscriber': {
            'username': 'test-member',
            'role': models.UserRole.admin.subscriber.name,
            'name': 'Test Member',
            'email': 'member@freedom-emails.com',
        },
        'non-existent': {'username': 'does-not-exist'},
    }

}


def parse_self_other(text):
    return text.lower() == 'my'


def create_user_in_db(db_engine, test_user: dict):
    return utils.create_test_user(
        db_engine=db_engine,
        username=test_user['username'],
        password=TEST_USER_PASSWORDS,
        role=test_user['role'],
        full_name=test_user.get('name'),
        email_address=test_user.get('email'),
    )


@bdd.given(parsers.parse('I am an {user_role} user'))
@bdd.given(parsers.parse('I am a {user_role} user'))
def jwt_token(user_role, api_base_uri, db_engine):

    requesting_user = create_user_in_db(db_engine, USER_TEMPLATES['requester'][user_role])
    return requests.post(
        url=f'{api_base_uri}/auth',
        json={
            'username': requesting_user['username'],
            'password': TEST_USER_PASSWORDS,
        }
    ).json()['access_token']


@bdd.given(parsers.parse('I try to create an {user_role} user'), target_fixture='user_response')
@bdd.given(parsers.parse('I try to create a {user_role} user'), target_fixture='user_response')
def create_new_user_response(user_role, jwt_token, api_base_uri):

    user_info = USER_TEMPLATES['target_data'][user_role]
    user_creation_request = {
        'username': user_info['username'],
        'password': TEST_USER_PASSWORDS,
        'role': user_role,
        'profile': {
            'fullName': user_info['name'],
            'email': user_info['email'],
        }
    }

    return requests.post(
        url=f'{api_base_uri}/api/users',
        json=user_creation_request,
        headers={
            'Authorization': f'JWT {jwt_token}',
        },
    )


@bdd.given(
    parsers.parse('I try to modify {self_classifier:self_other} {user_role} user', {'self_other': parse_self_other}),
    target_fixture='user_response')
def modify_user_response(self_classifier, user_role, jwt_token, api_base_uri, db_engine):

    existing_user_template = USER_TEMPLATES['target_data']['non-existent']
    if self_classifier:
        existing_user_template = USER_TEMPLATES['requester'][user_role]
        create_user_in_db(db_engine, existing_user_template)
    elif not self_classifier and not user_role == 'non-existent':
        existing_user_template = USER_TEMPLATES['target_data'][user_role]
        create_user_in_db(db_engine, existing_user_template)

    user_creation_request = {
        'username': existing_user_template['username'],
        'password': TEST_USER_PASSWORDS,
        'role': user_role,
        'profile': {
            'fullName': f'{existing_user_template["name"]}{TEST_NAME_MODIFIER}',
            'email': existing_user_template['email'],
        }
    }

    return requests.put(
        url=f'{api_base_uri}/api/users/{existing_user_template["username"]}',
        json=user_creation_request,
        headers={
            'Authorization': f'JWT {jwt_token}',
        },
    )


@bdd.given(parsers.parse('I try to create an {user_role} user with an {type_of_request} request'),
           target_fixture='user_response')
def invalid_create_new_user_response(user_role, type_of_request, jwt_token, api_base_uri):

    if type_of_request == 'non-json':
        return requests.post(
            url=f'{api_base_uri}/api/users',
            body='',
            headers={
                'Authorization': f'JWT {jwt_token}',
            },
        )

    return requests.post(
        url=f'{api_base_uri}/api/users',
        json={},
        headers={
            'Authorization': f'JWT {jwt_token}',
        },
    )


@bdd.given(parsers.parse('I try to modify an {user_role} user with an {type_of_request} request'),
           target_fixture='user_response')
def invalid_modify_user_response(user_role, type_of_request, jwt_token, api_base_uri):

    test_username = USER_TEMPLATES['target_data'][user_role]['username']

    if type_of_request == 'non-json':
        return requests.put(
            url=f'{api_base_uri}/api/users/{test_username}',
            body='',
            headers={
                'Authorization': f'JWT {jwt_token}',
            },
        )

    return requests.put(
        url=f'{api_base_uri}/api/users/{test_username}',
        json={},
        headers={
            'Authorization': f'JWT {jwt_token}',
        },
    )


@bdd.given(parsers.parse('I try to get {self_classifier:self_other} {user_role} user',
                         {'self_other': parse_self_other}),
           target_fixture='user_response')
def get_user_response(self_classifier, user_role, jwt_token, api_base_uri, db_engine):

    if self_classifier:
        existing_user = create_user_in_db(db_engine, USER_TEMPLATES['requester'][user_role])
    elif not self_classifier and not user_role == 'non-existent':
        existing_user = create_user_in_db(db_engine, USER_TEMPLATES['target_data'][user_role])
    else:
        existing_user = USER_TEMPLATES['target_data']['non-existent']

    return requests.get(
        url=f'{api_base_uri}/api/users/{existing_user["username"]}',
        headers={
            'Authorization': f'JWT {jwt_token}',
        },
    )


@bdd.given(parsers.parse('I try to delete {self_classifier:self_other} {user_role} user',
                         {'self_other': parse_self_other}),
           target_fixture='user_response')
def delete_user_response(self_classifier, user_role, jwt_token, api_base_uri, db_engine):

    if self_classifier:
        existing_user = create_user_in_db(db_engine, USER_TEMPLATES['requester'][user_role])
    elif not self_classifier and not user_role == 'non-existent':
        existing_user = create_user_in_db(db_engine, USER_TEMPLATES['target_data'][user_role])
    else:
        existing_user = USER_TEMPLATES['target_data']['non-existent']

    return requests.delete(
        url=f'{api_base_uri}/api/users/{existing_user["username"]}',
        headers={
            'Authorization': f'JWT {jwt_token}',
        },
    )


@bdd.then(parsers.parse('I get a new {user_role} user'))
def assert_create_user_response(user_role, user_response, api_base_uri):

    created_user = USER_TEMPLATES['target_data'][user_role]

    assert user_response.status_code == http.HTTPStatus.CREATED
    expected_user_creation_response = {
        'username': created_user['username'],
        'role': user_role,
        'profile': {
            'aliasName': None,
            'fullName': created_user.get('name'),
            'email': created_user.get('email'),
        },
        'urls': {
            'self': f'{api_base_uri}/api/users/{created_user["username"]}'
        }
    }
    assert user_response.json() == expected_user_creation_response


@bdd.then(parsers.parse('I get {self_classifier:self_other} {user_role} user',
                        {'self_other': parse_self_other}))
def assert_user_profile(self_classifier, user_role, user_response, db_engine, api_base_uri):

    template_user = USER_TEMPLATES['target_data'][user_role]
    if self_classifier:
        template_user = USER_TEMPLATES['requester'][user_role]

    create_user_in_db(db_engine, template_user)

    assert user_response.status_code == http.HTTPStatus.OK
    expected_user_creation_response = {
        'username': template_user['username'],
        'role': user_role,
        'profile': {
            'aliasName': None,
            'fullName': template_user.get('name'),
            'email': template_user.get('email'),
        },
        'urls': {
            'self': f'{api_base_uri}/api/users/{template_user["username"]}'
        }
    }
    assert user_response.json() == expected_user_creation_response


@bdd.then(parsers.parse('updates to {self_classifier:self_other} {user_role} user are preserved',
                        {'self_other': parse_self_other}))
def assert_modified_user_profile(self_classifier, user_role, user_response, api_base_uri):

    template_user = USER_TEMPLATES['target_data'][user_role]
    if self_classifier:
        template_user = USER_TEMPLATES['requester'][user_role]

    assert user_response.status_code == http.HTTPStatus.CREATED
    expected_user_creation_response = {
        'username': template_user['username'],
        'role': user_role,
        'profile': {
            'aliasName': None,
            'fullName': f'{template_user.get("name")}{TEST_NAME_MODIFIER}',
            'email': template_user.get('email'),
        },
        'urls': {
            'self': f'{api_base_uri}/api/users/{template_user["username"]}'
        }
    }
    assert user_response.json() == expected_user_creation_response


@bdd.then(parsers.parse('I get an unauthorized response'))
def assert_unauthorized_response(user_response: requests.Response):

    assert user_response.status_code == http.HTTPStatus.UNAUTHORIZED

    expected_response_json = {
        'error': {
            'status_code': http.HTTPStatus.UNAUTHORIZED.value,
            'message': 'Not authorized to access this resource.',
            'resource': user_response.request.url,
        }
    }
    assert user_response.json() == expected_response_json


@bdd.then(parsers.parse('I get an forbidden response'))
def assert_forbidden_response(user_response: requests.Response):

    assert user_response.status_code == http.HTTPStatus.FORBIDDEN

    expected_response_json = {
        'error': {
            'status_code': http.HTTPStatus.FORBIDDEN.value,
            'message': 'Can not delete your own user.',
            'resource': user_response.request.url,
        }
    }
    assert user_response.json() == expected_response_json


@bdd.then(parsers.parse('I get a user can not be found message'))
def assert_resource_not_found_response(user_response: requests.Response):
    assert user_response.status_code == http.HTTPStatus.NOT_FOUND

    expected_response_json = {
        'error': {
            'status_code': http.HTTPStatus.NOT_FOUND.value,
            'message': 'Resource not found.',
            'resource': user_response.request.url,
        },
    }
    assert user_response.json() == expected_response_json


@bdd.then(parsers.parse('I get a bad request response'))
def assert_bad_request_response(user_response: requests.Response):

    assert user_response.status_code == http.HTTPStatus.BAD_REQUEST

    expected_response_json = {
        'error': {
            'status_code': http.HTTPStatus.BAD_REQUEST.value,
            'message': mock.ANY,
        }
    }
    assert user_response.json() == expected_response_json


@bdd.then(parsers.parse('I get a response indicating the deletion was successful'))
def assert_resource_deleted_response(user_response: requests.Response):
    assert user_response.status_code == http.HTTPStatus.NO_CONTENT
    assert not user_response.content
    assert len(user_response.content) == 0
